<?xml version="1.0" encoding="UTF-8"?>
<xsl:stylesheet version="1.0"
                xmlns:xsl="http://www.w3.org/1999/XSL/Transform"
                xmlns:xalan="http://xml.apache.org/xalan"
                xmlns:str="http://exslt.org/strings">

<!--
     XSLT template for transforming the document used as internal
     representation of a ResultBean to VOTable v1.2
     Daniel Tapiador de Pedro - ESA/ESAC - Madrid, Spain
-->

<xsl:output method="text"/>
<xsl:template match="/">&lt;?xml version="1.0" encoding="UTF-8"?&gt;

&lt;VOTABLE version="1.2" xmlns:xsi="http://www.w3.org/2001/XMLSchema-instance"
 xmlns="http://www.ivoa.net/xml/VOTable/v1.2"&gt;

	&lt;RESOURCE name="ResultBean"&gt;

		&lt;TABLE name="Results"&gt;

			&lt;DESCRIPTION&gt;Data returned in the query&lt;/DESCRIPTION&gt;

			<!-- Get all attribute names -->
			<xsl:variable name="allNames">
				<xsl:for-each select="ResultBean/ResultBeanItem/item">
					<xsl:value-of select="@name"/>
					<xsl:value-of select="'-'"/>
					<xsl:value-of select="@type"/>
					<xsl:value-of select="'-'"/>
					<xsl:value-of select="@unit"/>
					<xsl:value-of select="'-'"/>
					<xsl:value-of select="' '"/>
				</xsl:for-each>
			</xsl:variable>

			<!-- Get the different attribute names -->
			<xsl:variable name="uniqueNames">
				<xsl:call-template name="build-name-list">
					<xsl:with-param name="list"><xsl:value-of select="$allNames"/></xsl:with-param>
				</xsl:call-template>
			</xsl:variable>

			<xsl:for-each select="str:tokenize($uniqueNames, ' ')">
				<xsl:call-template name="build-field">
					<xsl:with-param name="attrs"><xsl:value-of select="."/></xsl:with-param>
				</xsl:call-template>
			</xsl:for-each>

			&lt;DATA&gt;
				&lt;TABLEDATA&gt;

					<xsl:for-each select="ResultBean/ResultBeanItem">
						&lt;TR&gt;
							<xsl:call-template name="build-data">
								<xsl:with-param name="attrs"><xsl:value-of select="$uniqueNames"/></xsl:with-param>
							</xsl:call-template>
						&lt;/TR&gt;
					</xsl:for-each>

				&lt;/TABLEDATA&gt;
			&lt;/DATA&gt;

		&lt;/TABLE&gt;

	&lt;/RESOURCE&gt;

&lt;/VOTABLE&gt;

</xsl:template>

<xsl:template name="build-name-list">
	<xsl:param name="list" select="''"/>
	<xsl:choose>
		<xsl:when test="contains($list, ' ')">
			<xsl:variable name="token"><xsl:value-of select="substring-before($list,' ')"/></xsl:variable>
			<xsl:variable name="right"><xsl:value-of select="substring-after($list,' ')"/></xsl:variable>

			<xsl:choose>
				<xsl:when test="contains($right,$token)">
					<xsl:call-template name="build-name-list">
						<xsl:with-param name="list"><xsl:value-of select="$right"/></xsl:with-param>
					</xsl:call-template>
				</xsl:when>
				<xsl:otherwise>
					<xsl:value-of select="$token"/>
					<xsl:value-of select="' '"/>
					<xsl:call-template name="build-name-list">
						<xsl:with-param name="list"><xsl:value-of select="$right"/></xsl:with-param>
					</xsl:call-template>
				</xsl:otherwise>
			</xsl:choose>
		</xsl:when>
	</xsl:choose>
</xsl:template>

<xsl:template name="build-field">
	<xsl:param name="attrs" select="''"/>
	<xsl:param name="index" select="'1'"/>

	<xsl:variable name="token"><xsl:value-of select="substring-before($attrs,'-')"/></xsl:variable>
	<xsl:variable name="right"><xsl:value-of select="substring-after($attrs,'-')"/></xsl:variable>

	<xsl:choose>
		<xsl:when test="$index = 1">
			&lt;FIELD name="<xsl:value-of select="$token"/>"
			<xsl:call-template name="build-field">
				<xsl:with-param name="attrs"><xsl:value-of select="$right"/></xsl:with-param>
				<xsl:with-param name="index"><xsl:value-of select="$index + 1"/></xsl:with-param>
			</xsl:call-template>

		</xsl:when>
		<xsl:when test="$index = 2">
			<xsl:choose>
				<xsl:when test="$token = 'Integer'">
	   			datatype="int"
				</xsl:when>
				<xsl:when test="$token = 'Float'">
	   			datatype="float"
				</xsl:when>
				<xsl:when test="$token = 'Long'">
	   			datatype="long"
				</xsl:when>
				<xsl:when test="$token = 'Double'">
	   			datatype="double"
				</xsl:when>
				<xsl:when test="$token = 'String'">
	   			datatype="char" arraysize="*"
				</xsl:when>
				<xsl:when test="$token = 'Date'">
	   			datatype="char" arraysize="*"
				</xsl:when>
				<xsl:when test="$token = 'Boolean'">
	   			datatype="boolean"
				</xsl:when>
			</xsl:choose>
			<xsl:call-template name="build-field">
				<xsl:with-param name="attrs"><xsl:value-of select="$right"/></xsl:with-param>
				<xsl:with-param name="index"><xsl:value-of select="$index + 1"/></xsl:with-param>
			</xsl:call-template>
		</xsl:when>
		<xsl:when test="$index = 3">
	   			unit="<xsl:value-of select="$token"/>"/&gt;
		</xsl:when>
	</xsl:choose>
</xsl:template>

<xsl:template name="build-data">
	<xsl:param name="attrs" select="''"/>
		<xsl:choose>
			<xsl:when test="contains($attrs,'-')">
				<xsl:variable name="field"><xsl:value-of select="substring-before($attrs,' ')"/></xsl:variable>
				<xsl:variable name="token"><xsl:value-of select="substring-before($field,'-')"/></xsl:variable>
				<xsl:variable name="right"><xsl:value-of select="substring-after($attrs,' ')"/></xsl:variable>

				<xsl:variable name="valueEscaped">
					<xsl:call-template name="escape-value">
						<xsl:with-param name="value"><xsl:value-of select="item[@name=$token]/@value"/></xsl:with-param>
					</xsl:call-template>
				</xsl:variable>
							&lt;TD&gt;<xsl:value-of select="$valueEscaped"/>&lt;/TD&gt;

				<xsl:call-template name="build-data">
					<xsl:with-param name="attrs"><xsl:value-of select="$right"/></xsl:with-param>
				</xsl:call-template>
			</xsl:when>
		</xsl:choose>
</xsl:template>

<xsl:template name="escape-value">
	<xsl:param name="value" select="''"/>
		<xsl:choose>
			<xsl:when test="contains($value,'&lt;')">
				<xsl:variable name="left"><xsl:value-of select="substring-before($value,'&lt;')"/></xsl:variable>
				<xsl:variable name="right"><xsl:value-of select="substring-after($value,'&lt;')"/></xsl:variable>
				<xsl:value-of select="$left"/>&amp;lt;
				<xsl:call-template name="escape-value">
					<xsl:with-param name="value"><xsl:value-of select="$right"/></xsl:with-param>
				</xsl:call-template>
			</xsl:when>
			<xsl:otherwise>
				<xsl:value-of select="$value"/>
			</xsl:otherwise>
		</xsl:choose>
</xsl:template>

</xsl:stylesheet>
